from decouple import config
from .validators import validate_config

# Global state to hold loaded configuration (Singleton pattern)
_config_values = None


def load_config(custom_config=None) -> dict:
    """Load all configuration variables and validate them."""
    global _config_values

    if _config_values is None:
        default_config = {
            'LOG_LEVEL': config('LOG_LEVEL', default='INFO'),
            'DEBUG': config('DEBUG', default='false', cast=bool),
            'ENV': config('ENV', default='DEV')
        }

        # Merge custom configurator if present
        if custom_config:
            default_config.update(custom_config)

        config_values = {key: config(key, default=value) for key, value in default_config.items()}

        required_vars = [key for key, value in default_config.items() if value is None]
        validate_config(config_values, required_vars)

        _config_values = config_values

    return _config_values


def get_config(key: str, default=None):
    """Get a configuration value by key."""
    config_values = load_config()
    return config_values.get(key, default)
